@@ -2,8 +2,8 @@
 # 1.1   RMH    06-Feb-00     Extract a raw TTF table to a file
 
 use Font::TTF::Font;
-require 'getopts.pl';
-Getopts('at:');
+use Getopt::Std;
+getopts('at:');
 
 unless (defined $ARGV[0])
 {
@@ -18,8 +18,6 @@ to PC-compatible crlf, etc. (do this only on "text" tables such as TSIV).
 
 Example:  TTFExtractTable -tVTIS -a SomeFont.ttf out.txt
 
-Copyright (c) 2002 SIL International; All Rights Reserved.
-
 EOT
 }
 
