@@ -16,7 +16,7 @@ use Text::PDF::Utils;
 use Getopt::Std;
 use strict;
 use vars qw(@ISA %sizes @EXPORT $pdf_helv $pdf_helvb $pdf_helvi $pdf_helvbi $VERSION
-            $dots);
+            $dots %transformingops);
 
 require Exporter;
 @ISA = qw(Exporter);
@@ -41,6 +41,7 @@ BEGIN {
         require IO::Scalar;
     }
     $dots = "0 w [.25 1.25] 0 d";
+    %transformingops = map {$_=>1} qw(m l c v y);
 }
 
 
@@ -156,6 +157,7 @@ EOT
     { $font = Font::TTF::Font->open($ARGV[0]) || die "Can't open font file $ARGV[0]"; }
     $pdf->create_file($ARGV[1]);
     process_font($package, $font, $pdf, $root, $maxx, $maxy, "a0", %opt);
+    $package->final();
     $pdf->close_file;
 }
 
@@ -173,6 +175,8 @@ sub process_font
 #    $font->tables_do(sub { $_[0]->read; });
     @rev = $font->{'cmap'}->read->reverse;
 
+    my $mytime = $opt{'d'} ?  sub { return gmtime($_[0]); } : sub { return localtime($_[0]); };
+
     $numg = $font->{'maxp'}->read->{'numGlyphs'};
     $upem = $font->{'head'}->read->{'unitsPerEm'};
     $font->{'loca'}->read->glyphs_do(sub {
@@ -225,10 +229,10 @@ sub process_font
 no strict;        
     $ftrleft = "BT 1 0 0 1 36 27 Tm 80 Tz /FR 7 Tf (FRET v$VERSION "
         . "Package $package " . ${"${package}::VERSION"} . ") Tj ET\n";
-use strict;
-    @time = split(/\s+/, localtime($opt{d} || time()));
+    @time = split(/\s+/, &{$mytime}($opt{d} || time()));
     $tr = "Printed at $time[3] on $time[0] $time[2] $time[1] $time[4]   Page ";
-    @time = split(/\s+/, localtime($font->{'head'}->getdate));
+    @time = split(/\s+/, &{$mytime}($font->{'head'}->getdate));
+use strict;
     $hdrrw = "Modified at $time[3] on $time[0] $time[2] $time[1] $time[4]";
     $rpos = $maxx - 36 - $pdf_helv->width($hdrrw) * 5.6;
     $hdrlft .= "BT 1 0 0 1 $rpos ". ($maxy - 58) . " Tm 80 Tz /FR 7 Tf "
@@ -309,8 +313,10 @@ use strict;
                         && ($gcol = sprintf("%.2f %.2f %.2f rg",
                                             hex($1), hex($2), hex($3)));
                 $glyph = $font->{'loca'}{'glyphs'}[$gid];
+                my @parms = ($cids[$gcount + $i], $gid, $glyph, $rev[$gid], $font);
                 if ($glyph && $glyph->{' LEN'} != 0)
                 {
+                    my (@instr, $in);
                     $glyph->read;
                     $xorg = ($glyph->{'xMax'} + $glyph->{'xMin'}) * $tsize / $upem;
                     $xorg = $xcentre - $xorg / 2;
@@ -318,6 +324,21 @@ use strict;
                     $ppage->add(sprintf("%s %.4f %d m %.4f %d l S [] 0 d\n", $dots, $xadv, $ybase + 7, $xadv,$ybase + 57)) if ($xadv < $xcentre + 27);
                     $ppage->add(sprintf("%s %.4f %d m %.4f %d l S [] 0 d\n", $dots, $xorg, $ybase + 7, $xorg, $ybase + 57)) if ($xorg > $xcentre - 27 && $xorg < $xcentre + 27);
                     $ppage->add(sprintf("BT 1 0 0 1 %.4f %.4f Tm /T$id $tsize Tf 100 Tz $gcol <%04X> Tj %s ET\n", $xorg, $yorg, $gid, ($gcol ? "0 g " : "")));
+                    @instr = $package->overlay(@parms);
+                    if (@instr)
+                    {
+                        $ppage->add("q ");
+                        foreach $in (@instr)
+                        {
+                            if (defined $transformingops{$in->[0]})
+                            {
+                                for (my $j = 1; $j < @{$in}; $j++)
+                                { $in->[$j] = $in->[$j] * $tsize / $upem + ($j % 2 ? $xorg : $yorg); }
+                            }
+                            $ppage->add(join(' ', map {sprintf("%.4f", $_)} @{$in}[1..$#{$in}]) . " $in->[0] ");
+                        }
+                        $ppage->add("Q\n");
+                    }
                     unless ($opt{'r'})
                     {
                         $gxorg = ($glyph->{'xMax'} + $glyph->{'xMin'}) * $gsize / $upem / 2;
@@ -327,7 +348,6 @@ use strict;
                         $ppage->add(sprintf("BT 1 0 0 1 %.4f %.4f Tm /T$id $gsize Tf 100 Tz <%04X> Tj ET\n", $gxorg, $gyorg, $gid));
                     }
                 }
-                @parms = ($cids[$gcount + $i], $gid, $glyph, $rev[$gid], $font);
                 @rowt = $package->topdat(@parms);
                 @roww = widths(6, \@rowt);
                 @xorg = ([$xcentre - 26, $xcentre + 24 - $roww[1]],
@@ -537,6 +557,8 @@ sub get_points
     my ($onoff, $ends, $corners);
     my ($comp, $g);
 
+    if (!defined $glyph)
+    { return ([], [], [], undef); }
     $glyph->read_dat;
     if ($glyph->{'numberOfContours'} < 0)
     {
@@ -1007,22 +1029,42 @@ sub extra_points
     return [];
 }
 
+=head2 overlay
+
+Returns an array of arrays, each is an operator string followed by the parameters that will precede
+the operator. If the operator is a contour forming op, then the positions will be transformed
+from glyph space for rendering.
+
+=cut
+
+sub overlay
+{
+    my ($class, $cid, $gid, $glyph, $uid, $font) = @_;
+
+    return [];
+}
+
+=head2 final
+
+Called at the very end to tidy things up in the package
+
+=cut
+
+sub final
+{ }
+
+
 =head1 AUTHOR
 
-Martin Hosken L<Martin_Hosken@sil.org>. 
+Martin Hosken L<http://scripts.sil.org/FontUtils>. 
 
 =head1 LICENSING
 
-Copyright (c) 1998-2013, SIL International (http://www.sil.org)
-
-This module and all the various scripts are released under the terms of the
-Artistic License 2.0. For details, see the full text of the license in the file
-LICENSE.
-
-The test suite contains test fonts released under the Open Font License v1.1, see OFL.txt.
+Copyright (c) 1998-2016, SIL International (http://www.sil.org)
 
+This module is released under the terms of the Artistic License 2.0.
+For details, see the full text of the license in the file LICENSE.
 
 =cut
 
 1;
-
