@@ -91,6 +91,9 @@ map { $reserved{$_} = 1} (qw(
  
 sub start_afdko
 {
+    my ($self, $fh, %opts) = @_;
+    if ($opts{'preinclude'})
+    { $fh->print("include($opts{'preinclude'})" . ($opts{'z'} & 8 ? "" : ";") . "\n"); }
 }
 
 sub out_classes
@@ -128,7 +131,7 @@ sub out_classes
         }
         $fh->print("];\n\n");
 
-        next unless defined $vecs->{$l};
+        #next unless defined $vecs->{$l};
 
         $fh->print("\@cn${name}Dia = [");
         $count = 0; $sep = '';
@@ -147,8 +150,37 @@ sub out_classes
             { $sep = " "; }
         }
         $fh->print("];\n\n");
+        if ($name !~ /^Takes/o)
+        {
+            $fh->print("\@cMarkFilter_${name} = [\@c${name}Dia \@cTakes${name}Dia];\n");
+        }
     }
 
+    $fh->print("\@cGDEF_Bases = [");
+    $count = 0; $sep = '';
+    for ($c = 0; $c < $f->{'maxp'}{'numGlyphs'}; $c++)
+    {
+        next unless (vec($self->{'bases'}, $c, 1));
+        $fh->print("$sep$glyphs->[$c]{'name'}");
+        if (++$count % 8 == 0)
+        { $sep = "\n    "; }
+        else
+        { $sep = " "; }
+    }
+    $fh->print("];\n\n");
+
+    $fh->print("\@cGDEF_Attaches = [");
+    $count = 0; $sep = '';
+    for ($c = 0; $c < $f->{'maxp'}{'numGlyphs'}; $c++)
+    {
+        next unless (vec($self->{'ismarks'}, $c, 1));
+        $fh->print("$sep$glyphs->[$c]{'name'}");
+        if (++$count % 8 == 0)
+        { $sep = "\n    "; }
+        else
+        { $sep = " "; }
+    }
+    $fh->print("];\n\n");
 
     foreach $cl (sort {classcmp($a, $b)} keys %{$classes})
     {
@@ -214,9 +246,23 @@ sub out_pos_lookups
             my $b = \@bases;
             $b = \@mbases if ($mode);
             next if (!scalar @{$b});
-            my ($name) = "base_${l}_" . ($mode ? "mark" : "base");
+            my $type = $mode ? "mark" : "base";
+            my ($name) = "base_${l}_$type";
             $fh->print("lookup $name {\n");
-            $fh->print("  lookupflag 0;\n");
+            if ($mode)
+            {
+                if (defined $opts{'-m'}{$l})
+                {
+                    if ($opts{'-m'}{$l})
+                    { $fh->print("    lookupflag MarkAttachmentType \@$opts{'-m'}{$l};\n"); }
+                    else
+                    { $fh->print("    lookupflag 0;\n"); }
+                }
+                else
+                { $fh->print("  lookupflag UseMarkFilteringSet \@cMarkFilter_${l};\n"); }
+            }
+            else 
+            { $fh->print("  lookupflag 0;\n"); }
             foreach $c (@marks)
             {
                 my ($g) = $glyphs->[$c];
@@ -227,7 +273,7 @@ sub out_pos_lookups
             {
                 my ($g) = $glyphs->[$c];
                 my ($p) = $g->{'points'}{$l};
-                $fh->print("  pos base [$g->{'name'}] <anchor $p->{'x'} $p->{'y'}> mark \@$l;\n");
+                $fh->print("  pos $type [$g->{'name'}] <anchor $p->{'x'} $p->{'y'}> mark \@$l;\n");
             }
             $fh->print("} $name;\n\n");
         }
@@ -250,6 +296,11 @@ sub end_out
 
 1;
 
+=head1 NAME
+
+Font::TTF::Scripts::Fea - Creates font specific AFDKO fea source file from 
+                          a font and optional attachment point database
+
 =head1 See also
 
 L<Font::TTF::Scripts::AP>
@@ -262,7 +313,7 @@ Martin Hosken L<http://scripts.sil.org/FontUtils>.
 
 =head1 LICENSING
 
-Copyright (c) 1998-2014, SIL International (http://www.sil.org)
+Copyright (c) 1998-2016, SIL International (http://www.sil.org)
 
 This module is released under the terms of the Artistic License 2.0.
 For details, see the full text of the license in the file LICENSE.
