@@ -55,7 +55,7 @@ sub out_gdl
 
                 if ($opts{'-split_ligs'})
                 {
-                    if (defined $glyph->{'compunds'}{$pl})
+                    if (defined $glyph->{'compounds'}{$pl})
                     { $glyph->{'compounds'}{$pl}[3] = $pt->{'x'}; }
                     else
                     { $glyph->{'compounds'}{$pl} = [0, 0, $pt->{'x'}, $ytop]; }
@@ -142,7 +142,6 @@ sub out_classes
         else
         { $name =~ s/^_//o; }
 
-        $fh->print("#define HAS_c${name}Dia 1\n") if ($opts{'-defines'} && $name !~ m/^Takes/o);
         $fh->print("c${name}Dia = (");
         $count = 0; $sep = '';
         foreach $cl (@{$lists->{$l}})
@@ -160,14 +159,17 @@ sub out_classes
 
         $fh->print("cn${name}Dia = (");
         $count = 0; $sep = '';
+        $self->{'hasnclass'}{$l} = 0;
         for ($c = 0; $c < $f->{'maxp'}{'numGlyphs'}; $c++)
         {
             $psname = $f->{'post'}{'VAL'}[$c];
             next if ($psname eq '' || $psname eq '.notdef');
             next if (vec($vecs->{$l}, $c, 1));
+            next if (!(substr($l, 0, 1) eq "_") and vec($vecs->{"_$l"}, $c, 1));
             next if (defined $glyphs->[$c]{'props'}{'GDL_order'} && $glyphs->[$c]{'props'}{'GDL_order'} <= 1);
             next unless (vec($self->{'ismarks'}, $c, 1));
             $fh->print("$sep$glyphs->[$c]{'name'}");
+            $self->{'hasnclass'}{$l} = 1;
             if (++$count % 8 == 0)
             { $sep = ",\n    "; }
             else
@@ -175,6 +177,16 @@ sub out_classes
         }
         $fh->print(");\n\n");
     }
+    if ($opts{'-defines'})
+    {
+        foreach $l (sort keys %{$lists})
+        {
+            next if ($l =~ m/^_/o);
+            next if (!scalar @{$lists->{$l}} || !scalar @{$lists->{"_$l"}});
+            next if (!$self->{'hasnclass'}{$l} || !$self->{'hasnclass'}{"_$l"});
+            $fh->print("#define HAS_c${l}Dia 1\n");
+        }
+    }
 
 
     foreach $cl (sort {classcmp($a, $b)} keys %{$classes})
@@ -417,7 +429,9 @@ EOT
     foreach $p (keys %{$lists})
     {
         next if ($p =~ m/^_/o);
-        $fh->print("cTakes${p}Dia c${p}Dia {attach {to = \@1; at = ${p}S; with = ${p}M}; user1 = 1} / ^ _ opt4(cnTakes${p}Dia) _ {user1 == 0};\n");
+        next if (!scalar @{$lists->{$p}} || ref($lists->{"_$p"}) ne 'ARRAY' || !scalar @{$lists->{"_$p"}});
+        # $fh->print("cTakes${p}Dia c${p}Dia {attach {to = \@1; at = ${p}S; with = ${p}M}; user1 = 1} / ^ _ " . ($self->{'hasnclass'}{$p} ? "opt4(cnTakes${p}Dia) " : "") . "_ {user1 == 0};\n");
+        $fh->print("c${p}Dia {attach {to = \@1; at = ${p}S; with = ${p}M}} / cTakes${p}Dia " . ($self->{'hasnclass'}{$p} ? "opt4(cnTakes${p}Dia) " : "") . "_;\n");
     }
     $fh->print("endpass;\nendtable;\n");
 }
@@ -432,20 +446,20 @@ sub has
 
 1;
 
+=head1 NAME
+
+Font::TTF::Scripts::GDL - create GDL (Graphite Description Language) from a
+                          TrueType font
+
 =head1 AUTHOR
 
-Martin Hosken L<Martin_Hosken@sil.org>. 
+Martin Hosken L<http://scripts.sil.org/FontUtils>. 
 
 =head1 LICENSING
 
-Copyright (c) 1998-2013, SIL International (http://www.sil.org)
-
-This module and all the various scripts are released under the terms of the
-Artistic License 2.0. For details, see the full text of the license in the file
-LICENSE.
-
-The test suite contains test fonts released under the Open Font License v1.1, see OFL.txt.
+Copyright (c) 1998-2016, SIL International (http://www.sil.org)
 
+This module is released under the terms of the Artistic License 2.0.
+For details, see the full text of the license in the file LICENSE.
 
 =cut
-
