@@ -41,7 +41,7 @@ sub ttfname
         $fh->close();
     }
 
-    if (defined $opts{'l'} || ($opts{'t'} && (!$name->{'strings'}[$opts{'t'}] || !scalar @{$name->{'strings'}[$opts{'t'}]})))
+    if (defined $opts{'l'} || (defined $opts{'t'} && (!$name->{'strings'}[$opts{'t'}] || !scalar @{$name->{'strings'}[$opts{'t'}]})))
     {
         ## my ($cmap) = $font->{'cmap'}->read;
         ## @cover = map {[$_->{'Platform'}, $_->{'Encoding'}]} @{$cmap->{'Tables'}};
@@ -111,6 +111,9 @@ sub ttfname
 
 1;
 
+=head1 NAME
+
+Font::TTF::Scripts::Name - set name of font
 
 =head1 AUTHOR
 
@@ -118,7 +121,7 @@ Martin Hosken L<http://scripts.sil.org/FontUtils>.
 
 =head1 LICENSING
 
-Copyright (c) 1998-2014, SIL International (http://www.sil.org)
+Copyright (c) 1998-2016, SIL International (http://www.sil.org)
 
 This module is released under the terms of the Artistic License 2.0.
 For details, see the full text of the license in the file LICENSE.
