@@ -477,19 +477,19 @@ sub cut
 
 1;
 
+=head1 NAME
+
+Font::TTF::Scripts::Thai - handling of legacy (non-Unicode) Thai fonts
+
 =head1 AUTHOR
 
-Martin Hosken L<Martin_Hosken@sil.org>. 
+Martin Hosken L<http://scripts.sil.org/FontUtils>. 
 
 =head1 LICENSING
 
-Copyright (c) 1998-2013, SIL International (http://www.sil.org)
-
-This module and all the various scripts are released under the terms of the
-Artistic License 2.0. For details, see the full text of the license in the file
-LICENSE.
-
-The test suite contains test fonts released under the Open Font License v1.1, see OFL.txt.
+Copyright (c) 1998-2016, SIL International (http://www.sil.org)
 
+This module is released under the terms of the Artistic License 2.0.
+For details, see the full text of the license in the file LICENSE.
 
 =cut
