@@ -1138,7 +1138,7 @@ sub parse_volt
                 $res->{'lookups'}[-1]{'lookup'} = ['sub', [@content]];
 
                 unless ($str =~ m/\GEND_SUBSTITUTION\s+/ogc)
-                { die "Expected END_SUBSTITUION in LOOKUP $name, found: " . substr($str, pos($str), 20); }
+                { die "Expected END_SUBSTITUTION in LOOKUP $name, found: " . substr($str, pos($str), 20); }
             }
             else        # presume pos
             {
@@ -1808,17 +1808,14 @@ L<Font::TTF::Scripts::AP>
 
 =head1 AUTHOR
 
-Martin Hosken L<Martin_Hosken@sil.org>. 
+Martin Hosken L<http://scripts.sil.org/FontUtils>. 
 
 =head1 LICENSING
 
-Copyright (c) 1998-2013, SIL International (http://www.sil.org)
+Copyright (c) 1998-2016, SIL International (http://www.sil.org)
 
-This module and all the various scripts are released under the terms of the
-Artistic License 2.0. For details, see the full text of the license in the file
-LICENSE.
-
-The test suite contains test fonts released under the Open Font License v1.1, see OFL.txt.
+This module is released under the terms of the Artistic License 2.0.
+For details, see the full text of the license in the file LICENSE.
 
 
-=cut
+=cut
\ No newline at end of file
